<?php

defined('BOOTSTRAP') or die('Access denied');

use B2P\Responses\Error;
use B2P\Models\Interfaces\CreditOrder;

/**
 * @var string $mode
 */

$client = fn_paygine_get_client(fn_paygine_get_processor_params($mode));

try {
    $order_id = $_REQUEST['order_id'];
    if(!$order_id)
        throw new Exception(__('paygine.no_order_id'));

    $order_info = fn_get_order_info($order_id);
    $processor_data = fn_get_payment_method_data((int)$order_info['payment_id']);

    $pc_order_id = $order_info['payment_info']['order_id'];
    if(!$pc_order_id)
        throw new Exception(__('paygine.failed_get_order_id'));

    $ct_order = $client->order(['id' => $pc_order_id]);
    if($ct_order instanceof Error)
        throw new Exception($ct_order->description->getValue());

    if($mode === 'complete') {
        $complete_result = $ct_order->complete();
        if(!$complete_result)
            throw new Exception(__('paygine.failed_debit'));

        $pp_response['status'] = PAYGINE_ORDER_STATUS_COMPLETED;

        if($ct_order instanceof CreditOrder) {
            fn_set_notification('N', 'OK', __('paygine.loan_successful'));
        } else {
            if(!empty($processor_data['processor_params']['order_completed']))
                fn_change_order_status($order_id, $processor_data['processor_params']['order_completed']);

            $pp_response['order_status'] = fn_paygine_get_custom_order_status(PAYGINE_OPERATION_TYPE_COMPLETE, $processor_data['processor_params']);
            fn_set_notification('N', 'OK', __('paygine.complete_success_debit'));
        }

        fn_update_order_payment_info($order_info['order_id'], $pp_response);
    } elseif($mode === 'refund') {
        $reverse_result = $ct_order->reverse();
        if($reverse_result instanceof Error)
            throw new Exception(__('paygine.unable_refund'));

        if(!empty($processor_data['processor_params']['order_canceled']))
            fn_change_order_status($order_id, $processor_data['processor_params']['order_canceled']);

        fn_update_order_payment_info($order_info['order_id'], ['status' => PAYGINE_ORDER_STATUS_CANCELED, 'order_status' => fn_paygine_get_custom_order_status(PAYGINE_OPERATION_TYPE_REVERSE, $processor_data['processor_params'])]);
        fn_set_notification('N', 'OK', __('paygine.refund_completed'));
    }
} catch (Throwable $e) {
    fn_set_notification('E', __('paygine.error'), $e->getMessage());
}

return [CONTROLLER_STATUS_OK, 'orders.details?order_id=' . $order_id];