<?php

defined('BOOTSTRAP') or die('Access denied');

use B2P\Responses\Error;
use B2P\Models\Interfaces\CreditOrder;

/**
 * @var array $order_info
 * @var array $processor_data
 * @var string $mode
 */

$client = fn_paygine_get_client(fn_paygine_get_processor_params($mode));

if(defined('PAYMENT_NOTIFICATION')){
    if(($mode === 'success') || ($mode === 'fail')){
        try {
            if(isset($_REQUEST['error'])) {
                fn_set_notification('E', __('paygine.error'), __('paygine.redirect_error') . ":\n" . $_REQUEST['error']);
                fn_redirect('checkout.checkout');
            }

            if (!fn_check_payment_script(PAYGINE_PROCESSOR, (int)$_REQUEST['order_id']))
                die(__('paygine.access_denied'));

            if(!empty($_REQUEST['modal'])) {
                $_REQUEST['action'] = "payment_notification.{$mode}";
                fn_create_payment_form(fn_url('paygine.redirect'), $_REQUEST);
            }

            $order_id = (int)$_REQUEST['id'];
            if(!$order_id)
                throw new Exception(__('paygine.no_order_id'));

            $reference_id = (int)$_REQUEST['reference'];
            if(!$reference_id)
                throw new Exception(__('paygine.no_reference_id'));

            $operation_id = (int)$_REQUEST['operation'];
            if(!$operation_id)
                throw new Exception(__('paygine.unknown_operation'));

            $order_info = fn_get_order_info($reference_id);
            $processor_data = fn_get_payment_method_data((int)$order_info['payment_id']);

            $ct_order = $client->order(['id' => $order_id]);
            if($ct_order instanceof Error)
                throw new Exception($ct_order->description->getValue());

            $operationType = $ct_order->getOperation($operation_id)->type->getValue()->name;

            $pp_response['order_id'] = $order_id;
            $pp_response['status'] = $ct_order->getState();
            $pp_response['payment_type'] = $processor_data['processor_params']['payment_type'];
            $pp_response['currency'] = CART_PRIMARY_CURRENCY;
            $pp_response['amount'] = $ct_order->amount / 100;

            if($ct_order instanceof CreditOrder) {
                $pp_response['order_status'] = fn_paygine_get_custom_order_status($ct_order->isPaid() ? PAYGINE_OPERATION_TYPE_COMPLETE : PAYGINE_OPERATION_TYPE_LOAN, $processor_data['processor_params']);
            } else {
                if($ct_order->isPaid()) {
                    $pp_response['order_status'] = fn_paygine_get_custom_order_status($operationType, $processor_data['processor_params']);
                    fn_finish_payment((int)$ct_order->reference, $pp_response);
                } else {
                    fn_set_notification('E', __('paygine.error'), __('paygine.invalid_card'));
                    fn_redirect('checkout.checkout');
                }
            }
        } catch (Throwable $e) {
            fn_set_notification('E', __('paygine.error'), $e->getMessage());
        }

        fn_order_placement_routines('route', (int)$_REQUEST['order_id']);
    } elseif($mode == 'notify') {
        try {
            $response = file_get_contents("php://input");

            $ct_order_id = $client->handleResponse($response)->order_id->getValue();
            if(!$ct_order_id)
                throw new Exception(__('paygine.no_order_id'));

            $operation_id = $client->handleResponse($response)->id->getValue();
            if(!$operation_id)
                throw new Exception(__('paygine.unknown_operation'));

            $ct_order = $client->order(['id' => $ct_order_id]);
            if($ct_order instanceof Error)
                throw new Exception($ct_order->description->getValue());

            $ct_reference_id = $ct_order->reference;
            if(!$ct_reference_id)
                throw new Exception(__('paygine.no_reference_id'));

            $order_info = fn_get_order_info($ct_reference_id);
            $processor_data = fn_get_payment_method_data((int)$order_info['payment_id']);

            $operationType = $ct_order->getOperation($operation_id)->type->getValue()->name;

            $pp_response['order_id'] = $ct_order->id;
            $pp_response['status'] = $ct_order->getState();

            if($ct_order instanceof CreditOrder) {
                $pp_response['order_status'] = fn_paygine_get_custom_order_status($ct_order->isPaid() ? PAYGINE_OPERATION_TYPE_COMPLETE : PAYGINE_OPERATION_TYPE_LOAN, $processor_data['processor_params']);
            } else {
                if($ct_order->isPaid())
                    $pp_response['order_status'] = fn_paygine_get_custom_order_status($operationType, $processor_data['processor_params']);
            }

            if($ct_order->isPaid()) {
                fn_change_order_status($ct_reference_id, $pp_response['order_status']);
                fn_update_order_payment_info($ct_reference_id, $pp_response);
            }

            echo "ok";
        } catch (Throwable $e) {
            die($e->getMessage());
        }
    }
} else {
    $confirm_uri = 'payment_notification.success?payment=paygine&order_id=' . $order_info['order_id'];
    $cancel_uri = 'payment_notification.fail?payment=paygine&order_id=' . $order_info['order_id'];

    if(!empty($processor_data['processor_params']['modal_payform'])) {
        $confirm_uri .= "&modal=1";
        $cancel_uri .= "&modal=1";
    }

    [$fiscal_positions, $shop_cart] = fn_paygine_calc_fiscal_positions_shop_cart($client, $order_info);

    $register_data = [
        'reference' => $order_info['order_id'],
        'amount' => $client->centifyAmount($order_info['total']),
        'currency' => fn_paygine_get_currency(CART_PRIMARY_CURRENCY),
        'email' => $order_info['email'],
        'phone' => $order_info['phone'],
        'description' => 'Оплата заказа ' . $order_info['order_id'],
        'url' => fn_url($confirm_uri),
        'failurl' => fn_url($cancel_uri),
        'mode' => 0,
        'fiscal_positions' => $fiscal_positions
    ];

    try {
        $response = $client->register($register_data);
        if($response instanceof Error)
            throw new Exception($response->description->getValue());

        $b2p_order_id = (int)$response->id;
        if (!$b2p_order_id)
            throw new Exception(__('paygine.payment_process_error'));

        $operation_params = ['id' => $b2p_order_id];

        if (str_contains($processor_data['processor_params']['payment_type'], 'WithInstallment') && $shop_cart)
            $operation_params['shop_cart'] = base64_encode(json_encode($shop_cart));
        if (str_contains($processor_data['processor_params']['payment_type'], 'loan'))
            $operation_params['reference'] = $order_info['order_id'];

        $url = call_user_func([$client, $processor_data['processor_params']['payment_type']], $operation_params);
        
        if (!empty($processor_data['processor_params']['modal_payform']))
            fn_create_payment_form(fn_url('best2pay.modal'), [
                'host' => $client->getConfigManager()->getUrl(),
                'modal_url' => $url,
            ]);

        fn_redirect($url, true, true);
    } catch (Throwable $e) {
        fn_set_notification('E', __('paygine.error'), $e->getMessage());
        fn_redirect('checkout.checkout');
    }
}

exit;