<?php

namespace B2P\Models\Enums;

/**
 * Типы операций
 */
enum OperationType
{
    /**
     * Холдирование средств на счёте Платёжной карты Плательщика. Доступный
     * остаток средств на Платёжной карте Плательщика уменьшается, но
     * средства со счета не списываются до выполнения Операции COMPLETE
     */
    case AUTHORIZE;
    case AUTHORIZE_BY_TOKEN;

    /**
     * Операция подтверждения соответствующей Операции AUTHORIZE Заказа.
     * При успешном выполнении данной Операции производится списание
     * ранее захолдированных средств со счёта Платёжной карты Плательщика.
     */
    case COMPLETE;

    /**
     * Операция, при успешном выполнении которой сразу выполняется списание
     * средств со счёта Платёжной карты Плательщика. Данная Операция аналогична
     * последовательному выполнению пары Операций: AUTHORIZE и COMPLETE.
     */
    case PURCHASE;
    case PURCHASE_BY_TOKEN;

    /**
     * Операция оплаты QR-кодом СБП
     */
    case PURCHASE_BY_QR;

    /**
     * Операция, аналогичная PURCHASE (PURCHASE_BY_TOKEN). В отличие от
     * PURCHASE, проводится без участия Плательщика. Выполняется для
     * списания Рекуррентного платежа.
     */
    case RECURRING;
    case RECURRING_BY_TOKEN;

    /**
     * Возврат/Отмена. Позволяет вернуть средства, списанные или
     * захолдированные ранее в рамках Заказа.
     */
    case REVERSE;

    /**
     * Операция перевода средств с Карты на Карту.
     */
    case P2PTRANSFER;

    /**
     * Дебетовая часть перевода с Карты на Карту — списание денежных
     * средств с Карты отправителя.
     */
    case P2PDEBIT;

    /**
     * Кредитовая часть перевода с Карты на Карту — зачисление
     * денежных средств на Карту получателя.
     */
    case P2PCOMPLETE;

    /**
     * Операция зачисления денежных средств на Карту со счета ТСП.
     */
    case P2PCREDIT;

    /**
     * Операция Loan, инициируемая банком, подтверждает подписание кредитного договора.
     */
    case LOAN;
}