<?php

namespace B2P\Models\Enums;

/**
 * Статусы заказов
 */
enum OrderStatus
{
    /**
     * Заказ зарегистрирован в ПЦ и не содержит успешных Операций
     */
    case REGISTERED;

    /**
     * В рамках Заказа успешно проведена Операция типа AUTHORIZE (деньги для
     * проведения платежа заблокированы на счёте Карты Плательщика).
     */
    case AUTHORIZED;

    /**
     * В рамках Заказа успешно проведена операция типа P2PDEBIT (деньги для
     * проведения перевода с карты на карту списаны с Платёжной карты Плательщика).
     */
    case P2PAUTHORIZED;

    /**
     * Заказ успешно оплачен. В рамках Заказа проведена успешная Операция одного
     * из типов: PURCHASE, COMPLETE, P2PTRANSFER, P2PCOMPLETE, P2PCREDIT.
     */
    case COMPLETED;

    /**
     * Заказ отменен. В рамках Заказа проведена успешная Операция типа REVERSE.
     * Деньги по проведенной ранее в рамках Заказа оплате возвращены Плательщику.
     */
    case CANCELED;

    /**
     * Заказ заблокирован внутренними правилами ПЦ.
     * ТСП необходимо получить подтверждение от Оператора ПЦ, либо дождаться
     * смены статуса Заказа на другой, прежде чем принимать решение об исполнении
     * обязательств перед Плательщиком.
     */
    case BLOCKED;

    /**
     * Заказ с истекшим сроком действия. Время, установленное на оплату Заказа, истекло.
     */
    case EXPIRED;
}